////////////////////////////////////////////////////////////////////
//!
//! KS-10 Processor
//!
//! \brief
//!      AC Block
//!
//! \details
//!
//! \todo
//!
//! \file
//!      ac_block.v
//!
//! \author
//!      Rob Doyle - doyle (at) cox (dot) net
//!
////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2012 Rob Doyle
//
// This source file may be used and distributed without
// restriction provided that this copyright statement is not
// removed from the file and that any derivative work contains
// the original copyright notice and the associated disclaimer.
//
// This source file is free software; you can redistribute it
// and/or modify it under the terms of the GNU Lesser General
// Public License as published by the Free Software Foundation;
// version 2.1 of the License.
//
// This source is distributed in the hope that it will be
// useful, but WITHOUT ANY WARRANTY; without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General
// Public License along with this source; if not, download it
// from http://www.gnu.org/licenses/lgpl.txt
//
////////////////////////////////////////////////////////////////////
//
// Comments are formatted for doxygen
//

`default_nettype none
`include "useq/crom.vh"

module AC_BLOCK(clk, rst, clken, crom, dp, acBLOCK);

   parameter cromWidth = `CROM_WIDTH;

   input                  clk;          // Clock
   input                  rst;          // Reset
   input                  clken;        // Clock Enable
   input  [0:cromWidth-1] crom;         // Control ROM Data
   input  [0:35]          dp;           // Datapath
   output [0: 5]          acBLOCK;      // Blocks

   //
   // AC Block Selection
   //
   // Details
   //  This register holds the current AC block and the
   //  previous AC block.  The PXCT selects between the
   //  two.
   //
   // Trace
   //  DPE5/E308
   //

   wire specACBLK = `cromSPEC_EN_40 & (`cromSPEC_SEL == `cromSPEC_SEL_LOADACBLK);

   reg [0: 2] currBLOCK;                // Current Block
   reg [0: 2] prevBLOCK;                // Previous Block

   always @(posedge clk or posedge rst)
    begin
        if (rst)
          begin
             currBLOCK <= 3'b0;
             prevBLOCK <= 3'b0;
          end
        else if (clken & specACBLK)
          begin
             currBLOCK <= dp[6: 8];
             prevBLOCK <= dp[9:11];
          end
    end

   //
   // Fixups
   //

   assign acBLOCK[0:2] = currBLOCK;
   assign acBLOCK[3:5] = prevBLOCK;

 endmodule
