-------------------------------------------------------------------
--!
--! PDP-8 Processor
--!
--! \brief
--!      ASCII Definitions
--!
--! \details
--!      This package contains ASCII character constants.
--!
--! \file
--!      ascii_types.vhd
--!
--! \author
--!      Rob Doyle - doyle (at) cox (dot) net
--!
--------------------------------------------------------------------
--
--  Copyright (C) 2009, 2010, 2011, 2012 Rob Doyle
--
-- This source file may be used and distributed without
-- restriction provided that this copyright statement is not
-- removed from the file and that any derivative work contains
-- the original copyright notice and the associated disclaimer.
--
-- This source file is free software; you can redistribute it
-- and/or modify it under the terms of the GNU Lesser General
-- Public License as published by the Free Software Foundation;
-- version 2.1 of the License.
--
-- This source is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied
-- warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-- PURPOSE. See the GNU Lesser General Public License for more
-- details.
--
-- You should have received a copy of the GNU Lesser General
-- Public License along with this source; if not, download it
-- from http://www.gnu.org/licenses/lgpl.txt
--
--------------------------------------------------------------------
--
-- Comments are formatted for doxygen
--

library ieee;                                   --! IEEE Library
use ieee.std_logic_1164.all;                    --! IEEE 1164
    
--
--! Type definitions required for ASCII
--

package ascii_types is

    subtype  ascii_t    is std_logic_vector(0 to 7);
    
    constant ascii_LF   : ascii_t := x"0a";     --! CR
    constant ascii_CR   : ascii_t := x"0d";     --! LF
    constant ascii_DC1  : ascii_t := x"11";     --! DC1
    constant ascii_XON  : ascii_t := x"11";     --! XON (^Q)
    constant ascii_DC3  : ascii_t := x"13";     --! DC3
    constant ascii_XOFF : ascii_t := x"13";     --! XOFF (^S)
    constant ascii_ESC  : ascii_t := x"1b";     --! ESC
    constant ascii_0    : ascii_t := x"30";     --! '0'
    constant ascii_1    : ascii_t := x"31";     --! '1'
    constant ascii_2    : ascii_t := x"32";     --! '2'
    constant ascii_3    : ascii_t := x"33";     --! '3'
    constant ascii_4    : ascii_t := x"34";     --! '4'
    constant ascii_5    : ascii_t := x"35";     --! '5'
    constant ascii_6    : ascii_t := x"36";     --! '6'
    constant ascii_7    : ascii_t := x"37";     --! '7'
    constant ascii_8    : ascii_t := x"38";     --! '8'
    constant ascii_9    : ascii_t := x"39";     --! '9'
    constant ascii_COLN : ascii_t := x"3a";     --! ':'
    constant ascii_SEMI : ascii_t := x"3b";     --! ';'
    constant ascii_LT   : ascii_t := x"3c";     --! '<'
    constant ascii_EQ   : ascii_t := x"3d";     --! '='
    constant ascii_GT   : ascii_t := x"3e";     --! '>'
    constant ascii_QM   : ascii_t := x"3f";     --! '?'
    constant ascii_AT   : ascii_t := x"40";     --! '@'
    constant ascii_A    : ascii_t := x"41";     --! 'A'
    constant ascii_B    : ascii_t := x"42";     --! 'B'
    constant ascii_C    : ascii_t := x"43";     --! 'C'
    constant ascii_D    : ascii_t := x"44";     --! 'D'
    constant ascii_H    : ascii_t := x"48";     --! 'H'
    constant ascii_I    : ascii_t := x"49";     --! 'I'
    constant ascii_J    : ascii_t := x"4a";     --! 'J'
    constant ascii_K    : ascii_t := x"4b";     --! 'K'
    constant ascii_R    : ascii_t := x"52";     --! 'R'
    constant ascii_RBR  : ascii_t := x"5b";     --! '['
    constant ascii_f    : ascii_t := x"66";     --! 'f'
    constant ascii_l    : ascii_t := x"6c";     --! 'l'
    constant ascii_m    : ascii_t := x"6d";     --! 'm'
    constant ascii_p    : ascii_t := x"70";     --! 'p'
    constant ascii_s    : ascii_t := x"73";     --! 's'
    constant ascii_st   : ascii_t := x"74";     --! 't'
    constant ascii_u    : ascii_t := x"75";     --! 'u'
  
end ascii_types;
