--------------------------------------------------------------------
--!
--! PDP-8 Processor
--!
--! \brief
--!      This device synchronizes the input to a clock signal.
--!
--! \details
--!      TBD
--!
--! \file
--!      demet.vhd
--!
--! \author
--!      Rob Doyle - doyle (at) cox (dot) net
--!
--------------------------------------------------------------------
--
--  Copyright (C) 2011 Rob Doyle
--
-- This source file may be used and distributed without
-- restriction provided that this copyright statement is not
-- removed from the file and that any derivative work contains
-- the original copyright notice and the associated disclaimer.
--
-- This source file is free software; you can redistribute it
-- and/or modify it under the terms of the GNU Lesser General
-- Public License as published by the Free Software Foundation;
-- version 2.1 of the License.
--
-- This source is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied
-- warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-- PURPOSE. See the GNU Lesser General Public License for more
-- details.
--
-- You should have received a copy of the GNU Lesser General
-- Public License along with this source; if not, download it
-- from http://www.gnu.org/licenses/lgpl.txt
--
--------------------------------------------------------------------
--
-- Comments are formatted for doxygen
--

library ieee;                                   --! IEEE Library
use ieee.std_logic_1164.all;                    --! IEEE 1164

--
--! Signal Synchronization Primitive Entity
--

entity eDEMET is port (
    clk : in  std_logic;                     	--! Clock
    rst : in  std_logic;                     	--! Async Reset
    i   : in  std_logic;                     	--! In
    o   : out std_logic                     	--! Out
);
end eDEMET;

--
--! Signal Synchronization Primitive RTL
--

architecture rtl of eDEMET is
    
    signal temp : std_logic;             	--! Temp signal
    
begin

    --
    --! Synchronize input to clock signal
    --
    
    pDEMET : process (clk, rst)
    begin
        if rst = '1' then
            temp <= '0';
            o    <= '0';
        elsif (clk'event and clk = '1') then
            temp <= i;
            o    <= temp;
        end if;
    end process pDEMET;
    
end rtl;
